@@ -26,9 +26,8 @@
 #include "util.h"
 #include "debug.h"
 #include <assert.h>
-
 #include <sys/types.h>
-#include <dirent.h>
+#include "uce_dirent.h"
 
 #define TEMP_STR_LEN	(SR_MAX_PATH*2)
 
@@ -55,16 +54,17 @@ static void trim_filename (char *filename, char* out);
 static void trim_mp3_suffix (char *filename, char* out);
 static error_code filelib_open_for_write (FHANDLE* fp, char *filename);
 static void set_show_filenames (void);
+#if defined (commentout)
 static int is_absolute_path (char* fn);
+#endif
 static void parse_and_subst_dir (char* pattern_head, char* pattern_tail,
-				 char* opat_path, char* icy_name);
+				 char* opat_path);
 static void parse_and_subst_pat (char* newfile, TRACK_INFO* ti);
 static void set_default_pattern (BOOL get_separate_dirs, BOOL do_count);
 static error_code set_output_directory_new (char* output_pattern,
 					    char* output_directory,
 					    int get_separate_dirs,
-					    int get_date_stamp,
-					    char* icy_name
+					    int get_date_stamp
 					    );
 static error_code sr_getcwd (char* dirbuf);
 static error_code add_trailing_slash (char *str);
@@ -93,6 +93,7 @@ static char 	m_icy_name[SR_MAX_PATH];
 static char*	m_extension;
 static BOOL	m_do_individual_tracks;
 static char     m_session_datebuf[DATEBUF_LEN];
+static char     m_stripped_icy_name[SR_MAX_PATH];
 
 // For now we're not going to care. If it makes it good. it not, will know 
 // When we try to create a file in the path.
@@ -153,7 +154,12 @@ filelib_init (BOOL do_individual_tracks,
     m_show_name[0] = 0;
     m_do_show = do_show_file;
     m_do_individual_tracks = do_individual_tracks;
+
     sr_strncpy (m_icy_name, icy_name, SR_MAX_PATH);
+    sr_strncpy (m_stripped_icy_name, icy_name, SR_MAX_PATH);
+    debug_printf ("Stripping icy name...\n");
+    strip_invalid_chars (m_stripped_icy_name);
+    debug_printf ("Done stripping icy name.\n");
 
     switch (content_type) {
     case CONTENT_TYPE_MP3:
@@ -208,8 +214,7 @@ filelib_init (BOOL do_individual_tracks,
     set_output_directory_new (output_pattern,
 			      output_directory,
 			      get_separate_dirs,
-			      get_date_stamp,
-			      icy_name);
+			      get_date_stamp);
 
     sprintf (m_incomplete_directory, "%s%s%c", m_output_directory,
 	     "incomplete", PATH_SLASH);
@@ -258,8 +263,7 @@ error_code
 set_output_directory_new (char* output_pattern,
 			  char* output_directory,
 			  int get_separate_dirs,
-			  int get_date_stamp,
-			  char* icy_name
+			  int get_date_stamp
 			  )
 {
     error_code ret;
@@ -324,7 +328,7 @@ set_output_directory_new (char* output_pattern,
 
     /* Fill in %S and %d patterns */
     sprintf (pattern_head, "%s%s%s", device, cwd_path, odir_path);
-    parse_and_subst_dir (pattern_head, pattern_tail, opat_path, icy_name);
+    parse_and_subst_dir (pattern_head, pattern_tail, opat_path);
 
     /* In case there is no %A, no %T, etc., use the default pattern */
     if (!*pattern_tail) strcpy (pattern_tail, "%A - %T");
@@ -342,11 +346,10 @@ set_output_directory_new (char* output_pattern,
    from track to track: %A, %T, %a, %D, %q, or %Q. 
    If %S or %d appear before this, substitute in. */
 static void
-parse_and_subst_dir (char* pattern_head, char* pattern_tail,
-		     char* opat_path, char* icy_name)
+parse_and_subst_dir (char* pattern_head, char* pattern_tail, char* opat_path)
 {
     int opi = 0;
-    int phi = 0;
+    unsigned int phi = 0;
     int ph_base_len;
     int op_tail_idx;
 
@@ -386,7 +389,7 @@ parse_and_subst_dir (char* pattern_head, char* pattern_tail,
 	    continue;
 	case 'S':
 	    /* append stream name */
-	    strncat (pattern_head, icy_name, SR_MAX_BASE-phi);
+	    strncat (pattern_head, m_stripped_icy_name, SR_MAX_BASE-phi);
 	    phi = strlen (pattern_head);
 	    opi+=2;
 	    continue;
@@ -425,6 +428,7 @@ parse_and_subst_dir (char* pattern_head, char* pattern_tail,
     pattern_head[ph_base_len] = 0;
     debug_printf ("Got pattern head: %s\n", pattern_head);
     debug_printf ("Got opat tail:    %s\n", &opat_path[op_tail_idx]);
+
     strcpy (pattern_tail, &opat_path[op_tail_idx]);
 }
 
@@ -449,7 +453,7 @@ set_output_directory_old (char* output_directory,
 #else
 	/* I wish I could do something like "realpath()" here, but it 
 	 * doesn't have (e.g.) posix conformance. */
-	if (is_absolute_path (output_directory)) {
+	if (IS_ABSOLUTE_PATH (output_directory)) {
 	    strncpy (base_dir,output_directory,SR_MAX_PATH);
 	    debug_printf("Had absolute path\n");
 	} else {
@@ -483,7 +487,7 @@ set_output_directory_old (char* output_directory,
 	struct tm *theTime;
 	int time_len = 0;
 	int length_used = base_dir_len;
-	int length_available;
+	unsigned int length_available;
 	int rc;
 
 	if (base_dir_len > SR_MAX_BASE) {
@@ -666,6 +670,9 @@ filelib_write_cue(TRACK_INFO* ti, int secs)
 static void
 parse_and_subst_pat (char* newfile, TRACK_INFO* ti)
 {
+    char stripped_artist[SR_MAX_PATH];
+    char stripped_title[SR_MAX_PATH];
+    char stripped_album[SR_MAX_PATH];
 #define DATEBUF_LEN 50
     char temp[DATEBUF_LEN];
     char datebuf[DATEBUF_LEN];
@@ -673,16 +680,24 @@ parse_and_subst_pat (char* newfile, TRACK_INFO* ti)
     int nfi = 0;
     int done;
     char* pat = m_output_pattern;
-    debug_printf ("OUTPUT PATTERN:%s\n", m_output_pattern);
 
     /* Reserve 5 bytes: 4 for the .mp3 extension, and 1 for null char */
     int MAX_FILEBASELEN = SR_MAX_PATH-5;
 
+    debug_printf ("OUTPUT PATTERN:%s\n", m_output_pattern);
     strcpy (newfile, m_output_directory);
     opi = 0;
     nfi = strlen(newfile);
     done = 0;
 
+    /* Strip artist, title, album */
+    sr_strncpy (stripped_artist, ti->artist, SR_MAX_PATH);
+    sr_strncpy (stripped_title, ti->title, SR_MAX_PATH);
+    sr_strncpy (stripped_album, ti->album, SR_MAX_PATH);
+    strip_invalid_chars(stripped_artist);
+    strip_invalid_chars(stripped_title);
+    strip_invalid_chars(stripped_album);
+
     while (nfi < MAX_FILEBASELEN) {
 	debug_printf ("COMPOSING OUTPUT PATTERN:%s\n", newfile);
 	if (pat[opi] == '\0') {
@@ -723,13 +738,13 @@ parse_and_subst_pat (char* newfile, TRACK_INFO* ti)
 	    continue;
 	case 'a':
 	    /* album */
-	    strncat (newfile, ti->album, MAX_FILEBASELEN-nfi);
+	    strncat (newfile, stripped_album, MAX_FILEBASELEN-nfi);
 	    nfi = strlen (newfile);
 	    opi+=2;
 	    continue;
 	case 'A':
 	    /* artist */
-	    strncat (newfile, ti->artist, MAX_FILEBASELEN-nfi);
+	    strncat (newfile, stripped_artist, MAX_FILEBASELEN-nfi);
 	    nfi = strlen (newfile);
 	    opi+=2;
 	    continue;
@@ -743,7 +758,7 @@ parse_and_subst_pat (char* newfile, TRACK_INFO* ti)
 	    continue;
 	case 'T':
 	    /* title */
-	    strncat (newfile, ti->title, MAX_FILEBASELEN-nfi);
+	    strncat (newfile, stripped_title, MAX_FILEBASELEN-nfi);
 	    nfi = strlen (newfile);
 	    opi+=2;
 	    continue;
@@ -830,15 +845,89 @@ filelib_start (TRACK_INFO* ti)
     return filelib_open_for_write(&m_file, newfile);
 }
 
+static long
+get_file_size (char *filename)
+{
+    long len;
+    FILE* fp = fopen(filename, "r");
+    if (!fp) return 0;
+
+    if (fseek (fp, 0, SEEK_END)) {
+	fclose(fp);
+	return 0;
+    }
+
+    len = ftell (fp);
+    if (len < 0) {
+	fclose(fp);
+	return 0;
+    }
+
+    fclose (fp);
+    return len;
+}
+
+/*
+ * Added by Daniel Lord 29.06.2005 to only overwrite files with better 
+ * captures, modified by GCS to get file size from file system 
+ */
+static BOOL
+new_file_is_better (char *oldfile, char *newfile)
+{
+    long oldfilesize=0;
+    long newfilesize=0;
+
+#if defined (commentout)
+    test_file=open(oldfile, O_RDWR | O_EXCL);
+    oldfilesize=FileSize(test_file);
+#endif
+
+    oldfilesize = get_file_size (oldfile);
+    newfilesize = get_file_size (newfile);
+    
+    /*
+     * simple size check for now. Newfile should have at least 1Meg. Else it's
+     * not very usefull most of the time.
+     */
+    /* GCS: This isn't quite true for low bitrate streams.  */
+#if defined (commentout)
+    if (newfilesize <= 524288) {
+	debug_printf("NFB: newfile smaller as 524288\n");
+	return FALSE;
+    }
+#endif
+
+    if (oldfilesize == -1) {
+	/* make sure we get the file in case of errors */
+	debug_printf("NFB: could not get old filesize\n");
+	return TRUE;
+    }
+
+    if (oldfilesize == newfilesize) {
+	debug_printf("NFB: Size Match\n");
+	return FALSE;
+    }
+
+    if (newfilesize < oldfilesize) {
+	debug_printf("NFB:newfile bigger as oldfile\n");
+	return FALSE;
+    }
+
+    debug_printf ("NFB:oldfilesize = %li, newfilesize = %li, "
+		  "overwriting file\n", oldfilesize, newfilesize);
+    return TRUE;
+}
+ 
+
 // Moves the file from incomplete to complete directory
 // fullpath is an output parameter
 error_code
-filelib_end (TRACK_INFO* ti, BOOL over_write_existing, BOOL truncate_dup, 
+filelib_end (TRACK_INFO* ti, 
+	     enum OverwriteOpt overwrite,
+	     BOOL truncate_dup, 
 	     char *fullpath)
 {
     BOOL ok_to_write = TRUE;
-    BOOL file_exists = FALSE;
-    FHANDLE test_file;
     char newfile[TEMP_STR_LEN];
 
     if (!m_do_individual_tracks) return SR_SUCCESS;
@@ -853,31 +942,27 @@ filelib_end (TRACK_INFO* ti, BOOL over_write_existing, BOOL truncate_dup,
     mkdir_recursive (newfile, 0);
 
     // If we are over writing existing tracks
-    if (!over_write_existing) {
+    switch (overwrite) {
+    case OVERWRITE_ALWAYS:
+	ok_to_write = TRUE;
+	break;
+    case OVERWRITE_NEVER:
 	ok_to_write = FALSE;
-
-	// test if we can open the file we would be overwriting
-#if WIN32
-	OutputDebugString(newfile);
-	test_file = CreateFile(newfile, GENERIC_WRITE, FILE_SHARE_READ, 
-			       NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 
-			       NULL);
-#else
-	test_file = open(newfile, O_WRONLY | O_EXCL);
-#endif
-	file_exists = (test_file != INVALID_FHANDLE);
-	if (!file_exists)
-	    ok_to_write = TRUE;
-	else
-	    CloseFile(test_file);
+	break;
+    case OVERWRITE_LARGER:
+    default:
+	/* Smart overwriting -- only overwrite if new file is bigger */
+	ok_to_write = new_file_is_better (newfile, m_incomplete_filename);
+	break;
     }
 
     if (ok_to_write) {
-	// JCBUG -- clean this up
-	int x = DeleteFile(newfile);
-	x = MoveFile(m_incomplete_filename, newfile);
+	if (file_exists (newfile)) {
+	    DeleteFile (newfile);
+	}
+	MoveFile (m_incomplete_filename, newfile);
     } else {
-	if (truncate_dup && file_exists) {
+	if (truncate_dup && file_exists(m_incomplete_filename)) {
 	    TruncateFile(m_incomplete_filename);
 	}
     }
@@ -1032,6 +1117,7 @@ trim_mp3_suffix(char *filename, char* out)
     }
 }
 
+#if defined (commentout)
 static int
 is_absolute_path (char* fn)
 {
@@ -1050,12 +1136,13 @@ is_absolute_path (char* fn)
     }
     return 0;
 }
+#endif
 
 static void
 set_show_filenames (void)
 {
     if (!*m_show_name) return;
-    if (is_absolute_path(m_show_name)) {
+    if (IS_ABSOLUTE_PATH(m_show_name)) {
 	strcpy (m_cue_name, m_show_name);
     } else {
         snprintf (m_cue_name, SR_MAX_PATH, "%s/%s", 
@@ -1074,13 +1161,14 @@ get_next_sequence_number (char* fn_base)
     int seq;
     char dir_name[SR_MAX_PATH];
     char fn_prefix[SR_MAX_PATH];
+#if defined (WIN32)
     DIR* dp;
     struct dirent* de;
-
-#if defined (WIN32)
-    /* Not yet implemented */
-    return 0;
 #else
+    DIR* dp;
+    struct dirent* de;
+#endif
+
     /* Get directory from fn_base */
     while (fn_base[di]) {
 	if (ISSLASH(fn_base[di])) {
@@ -1088,14 +1176,41 @@ get_next_sequence_number (char* fn_base)
 	}
 	di++;
     }
-    dir_name[0] = '\0';
     strncpy (dir_name, fn_base, edi);
+    dir_name[edi] = '\0';
 
     /* Get fn prefix from fn_base */
     fn_prefix[0] = '\0';
     strcpy (fn_prefix, &fn_base[edi+1]);
 
+#if defined (WIN32)
     /* Look through directory for a filenames that match prefix */
+    debug_printf ("Trying to opendir: %s\n", dir_name);
+    if ((dp = opendir (dir_name)) == 0) {
+	return 0;
+    }
+    debug_printf ("dir:%s\nprefix:%s\n", dir_name, fn_prefix);
+    seq = 0;
+    while ((de = readdir (dp)) != 0) {
+	debug_printf ("Checking file for sequence number: %s\n", de->d_name);
+	if (strncmp(de->d_name, fn_prefix, strlen(fn_prefix)) == 0) {
+	    debug_printf ("Prefix match\n", de->d_name);
+	    if (isdigit(de->d_name[strlen(fn_prefix)])) {
+		int this_seq = atoi(&de->d_name[strlen(fn_prefix)]);
+		debug_printf ("Digit match:%c,%d\n", 
+			      de->d_name[strlen(fn_prefix)], this_seq);
+		if (seq <= this_seq) {
+		    seq = this_seq + 1;
+		}
+	    }
+	}
+    }
+    closedir (dp);
+    debug_printf ("Final sequence number found: %d\n",seq);
+    return seq;
+#else
+    /* Look through directory for a filenames that match prefix */
+    debug_printf ("Trying to opendir: %s\n", dir_name);
     if ((dp = opendir (dir_name)) == 0) {
 	return 0;
     }
