@@ -3,7 +3,7 @@
 
 #include "srtypes.h"
 
-#define SRVERSION	"1.61.9"
+#define SRVERSION	"1.61.11"
 
 #if defined (WIN32)
 #define SRPLATFORM      "windows"
@@ -51,7 +51,6 @@ typedef struct RIP_MANAGER_INFOst
 // Rip manager flags options
 #define OPT_AUTO_RECONNECT	0x00000001		// reconnect automatticly if dropped
 #define OPT_SEPERATE_DIRS	0x00000002		// create a directory named after the server
-#define OPT_OVER_WRITE_TRACKS	0x00000004		// should files in the complete directory be overwritten
 #define OPT_SEARCH_PORTS	0x00000008		// relay server should search for a open port
 #define OPT_MAKE_RELAY		0x00000010		// don't make a relay server
 #define OPT_COUNT_FILES		0x00000020		// add a index counter to the filenames
@@ -63,12 +62,11 @@ typedef struct RIP_MANAGER_INFOst
 #define OPT_TRUNCATE_DUPS	0x00001000		// truncate file in the incomplete directory already present in complete
 #define OPT_INDIVIDUAL_TRACKS	0x00002000		// should we write the individual tracks?
 
-#define OPT_FLAG_ISSET(flags, opt)	((flags & opt) > 0)
-#define OPT_FLAG_SET(flags, opt)	(flags =| opt)
+#define OPT_FLAG_ISSET(flags, opt)	    ((flags & opt) > 0)
+#define OPT_FLAG_SET(flags, opt)	    (flags =| opt)
 
 #define GET_AUTO_RECONNECT(flags)		(OPT_FLAG_ISSET(flags, OPT_AUTO_RECONNECT))
 #define GET_SEPERATE_DIRS(flags)		(OPT_FLAG_ISSET(flags, OPT_SEPERATE_DIRS))
-#define GET_OVER_WRITE_TRACKS(flags)		(OPT_FLAG_ISSET(flags, OPT_OVER_WRITE_TRACKS))
 #define GET_SEARCH_PORTS(flags)			(OPT_FLAG_ISSET(flags, OPT_SEARCH_PORTS))
 #define GET_MAKE_RELAY(flags)			(OPT_FLAG_ISSET(flags, OPT_MAKE_RELAY))
 #define GET_COUNT_FILES(flags)			(OPT_FLAG_ISSET(flags, OPT_COUNT_FILES))
@@ -94,7 +92,6 @@ typedef struct RIP_MANAGER_INFOst
 #define SET_TRUNCATE_DUPS(flags)		(OPT_FLAG_SET(flags, OPT_TRUNCATE_DUPS))
 #define SET_INDIVIDUAL_TRACKS(flags)		(OPT_FLAG_SET(flags, OPT_INDIVIDUAL_TRACKS))
 
-
 typedef struct RIP_MANAGER_OPTIONSst
 {
     char url[MAX_URL_LEN];		// url of the stream to connect to
@@ -112,12 +109,12 @@ typedef struct RIP_MANAGER_OPTIONSst
     u_long maxMB_rip_size;		// max number of megabytes that can by writen out before we stop
     u_short flags;			// all booleans logically OR'd together (see above)
     char useragent[MAX_USERAGENT_STR];	// optional, use a different useragent
-    //char dropstring[MAX_DROPSTRING_LEN];// optional dropstring to be searched in metainfo;if found: metainfo dropped
     SPLITPOINT_OPTIONS sp_opt;		// More options concerning splitpoint rules
     int timeout;			// timeout, in seconds, before a stalled connection is forcefully closed
     int dropcount;			// number of tracks at beginning of connection to always ignore
     CODESET_OPTIONS cs_opt;             // which codeset should i use?
     int count_start;                    // which number to start counting?
+    enum OverwriteOpt overwrite;	// overwrite file in complete?
 
 } RIP_MANAGER_OPTIONS;
 
@@ -143,4 +140,7 @@ error_code rip_manager_put_raw_data(char *buf, int size);
 char *client_relay_header_generate (int icy_meta_support);
 void client_relay_header_release (char *ch);
 
+char* overwrite_opt_to_string (enum OverwriteOpt oo);
+enum OverwriteOpt string_to_overwrite_opt (char* str);
+
 #endif //__RIP_MANANGER_H__
