@@ -530,15 +530,18 @@ strip_invalid_chars_stable(char *str)
 	*wstrp = replacement;
     }
 
-#if defined (WIN32)
-    /* Strip trailing periods on windows */
+    /* Strip trailing periods.
+       I used to do this only for WIN32, but now I understand that 
+       the user could be running cygwin/mingw32, or even mounting 
+       a remote win32 filesystem.  Thus, now I always do it. */
     for (wstrp = w_in + wcslen(w_in) - 1; wstrp >= 0; wstrp--) {
 	if (*wstrp == L'.')
 	    *wstrp = 0;
 	else
 	    break;
     }
-#else
+
+#if !defined (WIN32)
     /* Replace leading periods on unix */
     for (wstrp = w_in; *wstrp; wstrp++) {
 	if (*wstrp == L'.')
@@ -655,4 +658,3 @@ sr_strncpy(char* dst, char* src, int n)
     }
     dst[i] = 0;
 }
-
